<?php

namespace Teknisa\FIN\Modules\ContaPagar\Controllers;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Request\DataSet as Data;
use Teknisa\FIN\Modules\ContaPagar\Services\TitulosAPagarService as TitulosAPagarService;

/**
 * Class TitulosAPagarController
 * @package Teknisa\FIN\Modules\ContaPagar\Controllers
 */
class TitulosAPagarController extends TeknisaCrud{

    protected $dataSourceName = 'boletpagar_tipobaixapag_contcorr';
    protected $parameterBagColumns = array(
        'NRORG',
        'CDEMPRESA',
        'VRBAIXAPAGINI',
        'VRBAIXAPAGFIN',
        'VRBAIXAPAGTOT',
        'VRTOTALCNABPAGINI',
        'VRTOTALCNABPAGFIN',
        'VRTOTALCNABPAGTOT',
        'NRNOTAFISCINI',
        'NRNOTAFISCFIN',
        'NRNOTAFISCTOT',
        'VRFINALBPAGINI',
        'VRFINALBPAGFIN',
        'VRFINALBPAGTOT',
        'VRBOLPAGINI',
        'VRBOLPAGFIN',
        'VRBOLPAGTOT',
        'NRCNABPAGREMINI',
        'NRCNABPAGREMFIN',
        'NRCNABPAGREMTOT',
        'NRCNABPAGRETINI',
        'NRCNABPAGRETFIN',
        'NRCNABPAGRETTOT',
        'CDREDUCONT',
        'TCDREDUCONT',
        'CDOPERADOR',
        'CHAVEPIX',
        'CODEQRPIX'
    );

    protected $titulosAPagarService;

    /**
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param Environment $environment
     * @param TitulosAPagarService $titulosAPagarService
     */
    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment, TitulosAPagarService $titulosAPagarService)
    {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->titulosAPagarService = $titulosAPagarService;
    }

    private function buildTitulosAPagarRow($row){
        $completeRow = array();
        $completeRow['NRORG'] = $row['NRORG'];
        $completeRow['CDFILIAL'] = $row['CDFILIAL'];
        $completeRow['CDFILIRECE'] = $row['CDFILIRECE'];
        $completeRow['CDEMPRESARECE'] = $row['CDEMPRESARECE'];
        $completeRow['CDINSCESTARECE'] = $row['CDINSCESTARECE'];
        $completeRow['CDINSCMUNIRECE'] = $row['CDINSCMUNIRECE'];
        $completeRow['CDFORNECED'] = $row['CDFORNECED'];
        $completeRow['NRINSJURFORN'] = $row['NRINSJURFORN'];
        $completeRow['IDTPIJURFORN'] = $row['IDTPIJURFORN'];
        $completeRow['NRNOTAFISC'] = $row['NRNOTAFISC'];
        $completeRow['CDSERIE'] = $row['CDSERIE'];
        $completeRow['IDSTATUS'] = $row['IDSTATUS'];
        $completeRow['CDTIPOOPER'] = $row['CDTIPOOPER'];
        $completeRow['VRNOTAFISC'] = $row['VRNOTAFISC'];
        $completeRow['CDTPCTPAGAR'] = $row['CDTPCTPAGAR'];
        $completeRow['DSCONTAPAG'] = $row['DSCONTAPAG'];
        $completeRow['CDCLASSFINA'] = $row['CDCLASSFINA'];
        $completeRow['CDOPERACTPG'] = $row['CDOPERACTPG'];
        $completeRow['CHAVEPIX']  = isset($row['CHAVEPIX']) ? $row['CHAVEPIX'] : null;
        $completeRow['CODEQRPIX'] = isset($row['CODEQRPIX'])? $row['CODEQRPIX']: null;

        $completeRow['VRCONTA']  = isset($row['VRCONTA'])? $row['VRCONTA'] : $completeRow['VRNOTAFISC'];

        // Parâmetros Iniciais
        $completeRow['DATAATUAL']       = DateUtil::getDataAtual();
        $completeRow['DTATUAVENPAG']    = $this->titulosAPagarService->formataData($row['DTATUAVENPAG']);
        $completeRow['DTEMISSAO']       = $this->titulosAPagarService->formataData($row['DTEMISSAO']);
        $completeRow['DTENTRSAID']      = $this->titulosAPagarService->formataData($row['DTENTRSAID']);

        // Parâmetros Nota Fiscal

        $completeRow['NRINSJURNF']      = $completeRow['NRINSJURFORN'];
        $completeRow['IDTPIJURNF']      = $completeRow['IDTPIJURFORN'];
        $completeRow['CDEMPRESA']       = $completeRow['CDEMPRESARECE'];
        $completeRow['CDINSCESTA']      = $completeRow['CDINSCESTARECE'];
        $completeRow['CDINSCMUNI']      = $completeRow['CDINSCMUNIRECE'];
        $completeRow['VRPRODUTOS']      = $completeRow['VRNOTAFISC'];
        $completeRow['DTLIBERACAOFIN']  = $completeRow['DATAATUAL'];
        $completeRow['DTLANCAMENNF']    = $completeRow['DATAATUAL'];
        $completeRow['IDSISTEMA']       = 'PAG';
        $completeRow['IDBLOQUEADO']     = 'S';
        $completeRow['IDDOCFISCAL']     = 'N';
        $completeRow['IDDOCFINANC']     = 'S';
        $completeRow['IDENTRSAIDOP']    = 'E';
        $completeRow['IDDOCCONTAB']     = 'S';

        // Parâmetros Boleto
        $completeRow['NRLANBOLPAG']     = $this->titulosAPagarService->geraNovoCodigo($completeRow['NRORG'], 'BOLETPAGAR'.$completeRow['CDFILIRECE']);
        $completeRow['VRBOLPAG']        = $completeRow['VRCONTA'];
        $completeRow['VRFINALBPAG']     = $completeRow['VRCONTA'];
        $completeRow['DTVENBOLPAG']     = $completeRow['DTATUAVENPAG'];

        // Parâmetros Movimentação Fluxo
        $completeRow['DTMOVFLUXO']      = $completeRow['DTATUAVENPAG'];
        $completeRow['IDMOVFLUXO']      = ($completeRow['IDSTATUS'] == 'V')?'P':'E';

        // Parâmetros Conta
        $completeRow['NRSEQVENPAG']     = '01';
        $completeRow['NRPARCELAPAG']    = '00';
        $completeRow['DTORIGVENPAG']    = $completeRow['DTATUAVENPAG'];
        $completeRow['VRATUAPAG']       = $completeRow['VRCONTA'];
        $completeRow['VRORIGPAG']       = $completeRow['VRCONTA'];
        $completeRow['CDFILIBPAG']      = $completeRow['CDFILIRECE'];
        $completeRow['IDSTATPAG']       = 'A';
        $completeRow['QTDIASVENPAG']    = 0;
        $completeRow['CDFIMOVFLPAG']    = $completeRow['CDFILIRECE'];
        $completeRow['CDFILILANC']      = $this->environment->getCdFilial();

        return $completeRow;
    }

    /**
     * Gera Titulo a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function geraTituloAPagar(Row $row, Response $response)
    {
        try {
            // Dados do Registro
            $row = $row->getRow();
            $permiteParcelamento = isset($row["PARCELAMENTO"]);

            // Configura a row para enviar para o service
            $row = self::buildTitulosAPagarRow($row);

            // Salva Titulo a Pagar
            $this->titulosAPagarService->geraTituloAPagar($row, $response, $permiteParcelamento);

            // Registra Dados
            $response->addDataSet(new DataSet('geraTituloAPagar', ['nrlanbolpag' => $row['CDFILIBPAG'].$row['NRLANBOLPAG']]));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Titulos a Pagar
     * @param DataSet $dataSet
     * @param Response $response
     */
    public function geraTitulosAPagar(Data $dataset, Response $response){
        try {
            // Dados do Registro
            $rows = $dataset->getDataSet()->getRows();

            $permiteParcelamento = isset($rows[0]["PARCELAMENTO"]);

            // Configura as rows para enviar para o service
            foreach($rows as &$row){
                $row = self::buildTitulosAPagarRow($row);
            }

            // Salva Titulos a Pagar
            $this->titulosAPagarService->geraTitulosAPagar($rows, $response, $permiteParcelamento);

            // Registra Dados
            $response->addDataSet(new DataSet('salvaTituloAPagar', ['success' => 'S']));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Edita Titulo a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function editaTituloAPagar(Row $row, Response $response)
    {
        try {
            // Dados do Registro
            $rows = $row->getRow();

            // Parâmetros Iniciais
            $nrorg        = $rows['NRORG'];
            $cdfilial     = $rows['CDFILIAL'];
            $nrlanbolpag  = $rows['NRLANBOLPAG'];
            $cdreducont   = $rows['CDREDUCONT'];
            $cdreducont1  = $rows['CDREDUCONT'];
            $cdtpbaixapag = $rows['CDTPBAIXAPAG'];
            $cdbancbolp   = $rows['CDBANCBOLP'];
            $cdagenbolp   = $rows['CDAGENBOLP'];
            $cdccorbolp   = $rows['CDCCORBOLP'];
            $vrpemulbpag  = $rows['VRPEMULBPAG'];
            $vrpemorbpag  = $rows['VRPEMORBPAG'];
            $vrmultabpag  = $rows['VRMULTABPAG'];
            $vrmordiabpag = $rows['VRMORDIABPAG'];
            $dslinhbarpag = $rows['DSLINHBARPAG'];
            $cdbarrapag   = $rows['CDBARRAPAG'];
            $dsboletpag   = $rows['DSBOLETPAG'];
            $changedsbol  = $rows['__changeDsboletpag'];
            $chavepix     = $rows['CHAVEPIX'];
            $codeqrpix    = $rows['CODEQRPIX'];
            $idchavepix   = $rows['IDCHAVEPIX'];
            $dschavepix   = $rows['DSCHAVEPIX'];
            

            // Edita Titulo a Pagar
            $this->titulosAPagarService->editaTituloAPagar(
                $nrorg, $cdfilial, $nrlanbolpag, $cdreducont, $cdreducont1, $cdtpbaixapag, $cdbancbolp,
                $cdagenbolp, $cdccorbolp, $vrpemulbpag, $vrpemorbpag, $vrmultabpag, $vrmordiabpag,
                $dslinhbarpag, $cdbarrapag, $dsboletpag, $changedsbol, $chavepix, $codeqrpix, $idchavepix, $dschavepix
            );

            // Registra Dados
            $response->addDataSet(new DataSet('editaTituloAPagar', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Deleta Vencimento Titulo a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function deletaVencTituloPagar(Row $row, Response $response)
    {
        try {
            $row  = $row->getRow();

            $nrorg       = $row['NRORG'];
            $cdfilial    = $row['CDFILIAL'];
            $nrlanbolpag = $row['NRLANBOLPAG'];

            $this->titulosAPagarService->deletaVencTituloPagar($nrorg, $cdfilial, $nrlanbolpag, $response);

            $response->addDataSet(new DataSet('deletaVencTituloPagar', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Deleta Documento Titulo a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function deletaDocTituloPagar(Row $row, Response $response)
    {
        try {
            $row  = $row->getRow();

            $nrorg       = $row['NRORG'];
            $cdfilial    = $row['CDFILIAL'];
            $nrlanbolpag = $row['NRLANBOLPAG'];

            $this->titulosAPagarService->deletaDocTituloPagar($nrorg, $cdfilial, $nrlanbolpag);

            $response->addDataSet(new DataSet('deletaDocTituloPagar', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Titulo a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function atualizaTituloAPagar(Row $row, Response $response)
    {
        try {
            // Dados do Registro
            $rows = $row->getRow();

            // Parâmetros Iniciais
            $nrorg           = $rows['NRORG'];
            $cdfilial        = $rows['CDFILIAL'];
            $nrnotafisc      = $rows['NRNOTAFISC'];
            $cdserieentr     = $rows['CDSERIEENTR'];
            $idstatus        = $rows['IDSTATUS'];
            $cdtpctpagar     = $rows['CDTPCTPAGAR'];
            $cdclassfina     = $rows['CDCLASSFINA'];
            $nrlanctonf      = $rows['NRLANCTONF'];
            $vrnotafisc      = $rows['VRNOTAFISC'];
            $dscontapag      = $rows['DSCONTAPAG'];
            $dtemissao       = $this->titulosAPagarService->formataData($rows['DTEMISSAO']);
            $dtentrsaid      = $this->titulosAPagarService->formataData($rows['DTENTRSAID']);

            // Parâmetros Movimentação Fluxo
            $idmovfluxo      = ($idstatus == 'V')?'P':'E';

            // Salva Titulo a Pagar
            $this->titulosAPagarService->atualizaTituloAPagar(
                $nrorg, $cdfilial, $nrnotafisc, $dtemissao, $dtentrsaid, $nrlanctonf,
                $cdserieentr, $idstatus, $idmovfluxo, $cdclassfina,
                $cdtpctpagar, $vrnotafisc, $dscontapag, $response
            );

            // Registra Dados
            $response->addDataSet(new DataSet('atualizaTituloAPagar', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Prorroga Titulo a Pagar
     * @param Data $dataset
     * @param Response $response
     */
    public function prorrogaTituloAPagar(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();

            // Declaração de Variáveis
            $nrorg       = $boletos[0]['nrorg'];
            $dtvenbolpag = $boletos[0]['dtvenbolpag'];
            $cdmotipror  = $boletos[0]['cdmotipror'];
            $idtppror    = $boletos[0]['idtppror'];
            $nrdiasvenc  = $boletos[0]['nrdiasvenc'];
            $cdreducont  = isset($boletos[0]['cdreducont'])?$boletos[0]['cdreducont']:null;

            // Prorroga Título a Pagar
            $this->titulosAPagarService->prorrogaTituloAPagar($nrorg, $dtvenbolpag, $cdmotipror, $boletos[1], $idtppror, $nrdiasvenc, $cdreducont);

            // Registra Dados
            $response->addDataSet(new DataSet('prorrogaTituloAPagar', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Remessa Titulo a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function remessaTituloPagar(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            // Dados do Registro
            $row  = $row->getRow();

            // Declaração de Variáveis
            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $remessaboleto   = $row['remessaboleto'];
            $boletosmarcados = $row['boletosmarcados'];

            // Gera Remessa Título a Pagar
            $dsarqcnabpag = $this->titulosAPagarService->remessaTituloPagar($nrorg, $cdfilial, $remessaboleto, $boletosmarcados);

            $response->addDataSet(new DataSet('remessaTituloPagar', ['dsarqcnabpag' => $dsarqcnabpag]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Exclui Título a Pagar da Remessa
     * @param Row $row
     * @param Response $response
     */
    public function excluiTituloPagRemessa(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);

            $row  = $row->getRow();

            $nrorg      = $row[0]['nrorg'];
            $cdfilial   = $row[0]['cdfilial'];
            $cdoperador = $row[0]['cdoperador'];
            $boletos    = $row[1];

            $this->titulosAPagarService->excluiTituloPagRemessa($nrorg, $cdfilial, $cdoperador, $boletos);

            $response->addDataSet(new DataSet('excluiTituloPagRemessa', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Exclui Remessa Pagar
     * @param Row $row
     * @param Response $response
     */
    public function excluiRemessaPag(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);

            $row  = $row->getRow();

            $nrorg      = $row[0]['nrorg'];
            $cdfilial   = $row[0]['cdfilial'];
            $cdoperador = $row[0]['cdoperador'];
            $boletos    = $row[1];

            $this->titulosAPagarService->excluiRemessaPag($nrorg, $cdfilial, $cdoperador, $boletos);

            $response->addDataSet(new DataSet('excluiRemessaPag', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Baixa Titulo a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function baixaTituloPagar(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $row  = $row->getRow();

            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $baixaboleto     = $row['baixaboleto'];
            $boletosmarcados = $row['boletosmarcados'];
            $ajustesboletos  = $row['ajustesboletos'];
            $contabiliza     = isset($row['contabiliza']) ? $row['contabiliza'] : true;
            $boletostotais   = isset($row['boletostotais']) ? $row['boletostotais'] : [];
            
            $this->titulosAPagarService->baixaTituloPagar($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $ajustesboletos, $contabiliza, $response, $boletostotais);

            $response->addDataSet(new DataSet('baixaTituloPagar', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Exclui Baixa a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function excluiBaixaPagar(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $row  = $row->getRow();

            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $boletosmarcados = $row['boletosmarcados'];
            $cheque          = $row['cheque'];

            $this->titulosAPagarService->excluiBaixaPagar($nrorg, $cdfilial, $boletosmarcados, $cheque);

            $response->addDataSet(new DataSet('excluiBaixaPagar', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Exclui Titulo Baixa a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function excluiTituloBaixaPagar(Row $row, Response $response)
    {
        try {
            $row  = $row->getRow();

            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $boletosmarcados = $row['boletosmarcados'];

            $this->titulosAPagarService->excluiTituloBaixaPagar($nrorg, $cdfilial, $boletosmarcados, $response);

            $response->addDataSet(new DataSet('excluiTituloBaixaPagar', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Exclui Baixa a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function transformaBaixaPlanejado(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $row  = $row->getRow();

            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $baixaboleto     = $row['baixaboleto'];
            $boletosmarcados = $row['boletosmarcados'];
            $cheque          = $row['cheque'];

            $this->titulosAPagarService->transformaBaixaPlanejado($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $cheque, $response);

            $response->addDataSet(new DataSet('excluiBaixaPagar', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Exclui Titulo Baixa a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function transformaTituloPlanejado(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $row  = $row->getRow();

            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $baixaboleto     = $row['baixaboleto'];
            $boletosmarcados = $row['boletosmarcados'];

            $this->titulosAPagarService->transformaTituloPlanejado($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response);

            $response->addDataSet(new DataSet('excluiTituloBaixaPagar', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Valida Associação Título
     * @param Data $dataset
     * @param Response $response
     */
    public function validaAssociacaoTitulo(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();

            $this->titulosAPagarService->validaAssociacaoTitulo($boletos);

            // Registra Dados
            $response->addDataSet(new DataSet('validaAssociarTitulo', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Confirma Associação Titulo
     * @param Data $dataset
     * @param Response $response
     */
    public function confirmaAssociacaoTitulo(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();

            $nrorg        = $boletos[0]['nrorg'];
            $cdfilial     = $boletos[0]['cdfilial'];
            $dtvenbolpag  = $boletos[0]['dtvenbolpag'];
            $cdreducont   = $boletos[0]['cdreducont'];
            $cdtpbaixapag = $boletos[0]['cdtpbaixapag'];
            $boletos      = $boletos[1];

            // Valida Data de Vencimento
            $this->titulosAPagarService->validaDataVencimento($dtvenbolpag, $boletos);

            // Gera Associação Título
            $this->titulosAPagarService->geraAssociacaoTitulo($nrorg, $cdfilial, $boletos, $dtvenbolpag, $cdreducont, $cdtpbaixapag);

            // Registra Dados
            $response->addDataSet(new DataSet('confirmaAssociacaoTitulo', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Cancela Associação Titulo a Pagar
     * @param Data $dataset
     * @param Response $response
     */
    public function cancelaAssociacaoTitulo(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();

            // Cancela Associação Título
            $this->titulosAPagarService->cancelaAssociacaoTitulo($boletos);

            // Registra Dados
            $response->addDataSet(new DataSet('cancelaAssociarTituloAPag', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Adiantamento Fornecedor
     * @param Data $dataset
     * @param Response $response
     */
    public function geraAdiantamentoFornecedor(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $tituloGrid         = $rows[0];
            $adiantamentoTabbed = $rows[1];
            $adiantamentoGrid   = $rows[2];
            
            $cdoperador = $this->environment->getCdOperador();

            // Gera Aditamento Fornencedor
            $this->titulosAPagarService->geraAdiantamentoFornecedor(
                $tituloGrid, $adiantamentoTabbed, $adiantamentoGrid, $cdoperador
            );

            // Registra Dados
            $response->addDataSet(new DataSet('geraAdiantamentoFornecedor', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Devolução Fornecedor
     * @param Data $dataset
     * @param Response $response
     */
    public function geraDevolucaoFornecedor(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $tituloGrid      = $rows[0];
            $devolucaoTabbed = $rows[1];
            $devolucaoGrid   = $rows[2];
            
            $cdoperador = $this->environment->getCdOperador();

            // Gera Devolução Fornencedor
            $this->titulosAPagarService->geraDevolucaoFornecedor(
                $tituloGrid, $devolucaoTabbed, $devolucaoGrid, $cdoperador
            );

            // Registra Dados
            $response->addDataSet(new DataSet('geraDevolucaoFornecedor', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Novo Código
     * @param Row $row
     * @param Response $response
     */
    public function geraNovoCodigo(Row $row, Response $response)
    {
        try {
            // Dados do Registro
            $row  = $row->getRow();

            $nrorg      = $row['NRORG'];
            $cdcontador = $row['CDCONTADOR'];

            $novocodigo = $this->titulosAPagarService->geraNovoCodigo($nrorg, $cdcontador);

            $response->addDataSet(new DataSet('geraNovoCodigo', ['novocodigo' => $novocodigo]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Cadastramento Automatico
     * @param Row $row
     * @param Response $response
     */
    public function geraCadastramentoAutomatico(Row $row, Response $response){
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $row  = $row->getRow();

            // Gera Data Source Cadastramento Automatico
            $rows = self::geraDataSourceCadastramentoAutomatico($row);

            // Configura as rows para enviar para o service
            foreach($rows as &$row){
                $row = self::buildTitulosAPagarRow($row);
            }

            // Gera Títulos a Pagar
            $this->titulosAPagarService->geraTitulosAPagar($rows, $response, false);

            // Registra Dados
            $response->addDataSet(new DataSet('geraCadastramentoAutomatico', ['success' => 'S']));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Salva Conta Documento
     * @param Row $row
     * @param Response $response
     */
    public function salvaContaDocumento(Row $row, Response $response)
    {
        try {
            $conta  = $row->getRow();
            $this->titulosAPagarService->salvaContaDocumento($conta);

            $response->addDataSet(new DataSet('salvaContaDocumento', ['success' => 'S']));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Data Source Cadastramento Automático
     * @param $row
     * @return array
     * @throws \Exception
     */
    public function geraDataSourceCadastramentoAutomatico($row){
        $rows = [];
        try {
            $row['NRLICENCAUSO'] = null;
            $row['CDVENDEDOR']   = null;
            $row['DSDEFACTPAG']  = isset($row['DSDEFACTPAG'])?$row['DSDEFACTPAG']:$row['DSTPCTPAGAR'];
            $row['DSCONTAPAG']   = isset($row['DSCONTAPAG'])?$row['DSCONTAPAG']:$row['DSDEFACTPAG'];

            for($i = 0; $i < $row['NRTITULOS']; $i++) {
                $row['NRNOTAFISC']   = $this->titulosAPagarService->geraNovoCodigo($row['NRORG'], 'NRNOTAFISCPAG', 1, $row['NRTAMNUMNFUTDFE']);
                $row['DTEMISSAO']    = $row['DTPRIMVENC'];
                $row['DTENTRSAID']   = $row['DTPRIMVENC'];
                $row['DTATUAVENPAG'] = $row['DTPRIMVENC'];

                $rows[] = clone $row;
                
                if (strlen($row['DTPRIMVENC']) == 19)
                    $row['DTPRIMVENC'] = DateUtil::truncateString($row['DTPRIMVENC']);

                $row['DTPRIMVENC'] = (($row['IDPROXVENC'] == 'F')
                    ? DateUtil::truncateString(DateUtil::adicionaIntervalo(DateUtil::truncateData(DateUtil::getDataDeString($row['DTPRIMVENC'])), 1, 'M'))
                    : DateUtil::truncateString(DateUtil::adicionaIntervalo(DateUtil::truncateData(DateUtil::getDataDeString($row['DTPRIMVENC'])), $row['NRDIASVENC'], 'D')));
            }
        } catch (\Exception $e) {
            throw $e;
        }
        return $rows;
    }

    /**
     * Finaliza Antecipação de Fornecedor
     * @param Data $dataset
     * @param Response $response
     */
    public function finalizaAntecipacaoFornecedor(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $rows  = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $antecipacao = $rows[0];
            $titulos     = $rows[1];

            // Gera Data Source Antecipação
            $antecipacao = $this->geraDataSourceAntecipacao($antecipacao);
            
            // Finaliza Antecipação de Fornecedor
            $this->titulosAPagarService->finalizaAntecipacaoFornecedor($antecipacao, $titulos);

            // Registra Dados
            $response->addDataSet(new DataSet('finalizaAntecipacaoFornecedor', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Data Source Antecipação
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public function geraDataSourceAntecipacao(
        $antecipacao
    ){
        $antecipacao['CDREDUCONTPAG']   = isset($antecipacao['CDREDUCONTPAG']  ) ? $antecipacao['CDREDUCONTPAG']   : null;
        $antecipacao['VRPEMULANTECIPA'] = isset($antecipacao['VRPEMULANTECIPA']) ? $antecipacao['VRPEMULANTECIPA'] : null;
        $antecipacao['VRPEMORAANTECIP'] = isset($antecipacao['VRPEMORAANTECIP']) ? $antecipacao['VRPEMORAANTECIP'] : null;
        $antecipacao['VRMULTAANTECIPA'] = isset($antecipacao['VRMULTAANTECIPA']) ? $antecipacao['VRMULTAANTECIPA'] : null;
        $antecipacao['VRMORAANTECIPA']  = isset($antecipacao['VRMORAANTECIPA'] ) ? $antecipacao['VRMORAANTECIPA']  : null;
        $antecipacao['DSLINHBARANTECI'] = isset($antecipacao['DSLINHBARANTECI']) ? $antecipacao['DSLINHBARANTECI'] : null;
        $antecipacao['CDBARRAANTECIPA'] = isset($antecipacao['CDBARRAANTECIPA']) ? $antecipacao['CDBARRAANTECIPA'] : null;
        $antecipacao['DSANTECIPAFORN']  = isset($antecipacao['DSANTECIPAFORN'] ) ? $antecipacao['DSANTECIPAFORN']  : null;
        $antecipacao['CDEMPRESA']       = isset($antecipacao['CDEMPRESA']      ) ? $antecipacao['CDEMPRESA']       : null;
        $antecipacao['DTEMISANTECIPA']  = isset($antecipacao['DTEMISANTECIPA'] ) ? $antecipacao['DTEMISANTECIPA']  : null;
        $antecipacao['CDBANCANTECIPA']  = isset($antecipacao['CDBANCANTECIPA'] ) ? $antecipacao['CDBANCANTECIPA']  : null;
        $antecipacao['CDAGENANTECIPA']  = isset($antecipacao['CDAGENANTECIPA'] ) ? $antecipacao['CDAGENANTECIPA']  : null;
        $antecipacao['CDCCORANTECIPA']  = isset($antecipacao['CDCCORANTECIPA'] ) ? $antecipacao['CDCCORANTECIPA']  : null;

        return $antecipacao;
    }

   /**
     * Cancela Antecipação de Fornecedores
     * @param Data $dataset
     * @param Response $response
     */
    public function cancelaAntecipacaoFornecedor(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $titulos = $rows[0];

            $this->titulosAPagarService->cancelaAntecipacaoFornecedor($titulos);

            // Registra Dados
            $response->addDataSet(new DataSet('cancelaAntecipacaoFornecedor', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Valida Filial Logada Po Empresa Filtrada
     * @param Row $row
     * @param Response $response
     */
    public function validaFilialLogadaPorEmpresaFiltrada(Row $row, Response $response){
        try {
            $row           = $row->getRow();
            $cdempresa     = $row['CDEMPRESA'];
            $cdfilial      = $this->environment->getCdFilial();
            $filialEmpresa = $this->titulosAPagarService->validaFilialLogadaPorEmpresaFiltrada($cdempresa, $cdfilial);
            $filialEmpresa = ($filialEmpresa) ? [$filialEmpresa] : [];
            $response->addDataSet(new DataSet('filial', $filialEmpresa));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Busca Todos Ajustes Boletos
     * @param Data $dataset
     * @param Response $response
     */
    public function buscaTodosAjustesBoletos(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();

            $ajustesboletos = $this->titulosAPagarService->buscaTodosAjustesBoletos($boletos);
               
            $response->addDataSet(new DataSet('buscaTodosAjustesBoletos', $ajustesboletos));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Gera Titulo a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function geraTituloAPagarDoc(Data $dataset, Response $response)
    {
        try {
            
            $dataset  = $dataset->getDataSet()->getRows();
            
            // Dados do Registro
            $row = $dataset[0];
            $anexos = $dataset[2];
            $permiteParcelamento = isset($row["PARCELAMENTO"]);
            
            $dataset  = $dataset[1];
            $row['DTATUAVENPAG'] = $dataset[0]['DTVENBOLPAG'];
            $row['DSCONTAPAG']   = $dataset[0]['DSCONTAPAG'];
            
            // Configura a row para enviar para o service
            $row = self::buildTitulosAPagarRow($row);
            
            
            
            if(count($dataset) > 1){
    
                // Variáveis de Inicialização
                $nrorg       = null;
                $cdfilial    = null;
                $nrlanbolpag = null;
                $parcelas = array();
    
                // Percorre Parcelas
                foreach($dataset as $data){
                    // Verifica Parcela Inicial
                    if(($data['NRPARCBOLPAG'] == '00') || ($data['NRPARCBOLPAG'] == '01')) {
                        $nrorg       = $row['NRORG'];
                        $cdfilial    = $row['CDFILIRECE'];
                        $nrlanbolpag = $row['NRLANBOLPAG'];
                    }
    
                    // Monta Array Parcelas
                    $parcelas[] = array(
                        'NRORG'        => $row['NRORG']        ,
                        'CDFILIAL'     => $row['CDFILIRECE']   ,
                        'NRLANBOLPAG'  => null                 ,
                        'NRORIGBOLPAG' => $row['NRLANBOLPAG']  ,
                        'NRPARCBOLPAG' => $data['NRPARCBOLPAG'],
                        'NRPARCBOLPAGVISU' => $data['NRPARCBOLPAGVISU'],
                        'DTVENBOLPAG'  => $data['DTVENBOLPAG'] ,
                        'DSCONTAPAG'   => $data['DSCONTAPAG']  ,
                        'VRFINALBPAG'  => floatval($data['VRFINALBPAG']),
                        'CDMOTIPARC'   => isset($data['CDMOTIPARC'])?$data['CDMOTIPARC']:null
                    );
                }
                
                // Salva Titulo a Pagar e Gera Parcelamento Título
                $this->titulosAPagarService->geraTituloAPagarDoc(
                    $row, $response, $permiteParcelamento, $nrorg, $cdfilial, $nrlanbolpag, $parcelas
                );
            }else{
                // Salva Titulo a Pagar
                $this->titulosAPagarService->geraTituloAPagar(
                    $row, $response, $permiteParcelamento
                );
            }
            
            if(!empty($anexos)){
                $this->titulosAPagarService->salvaAnexoInFileServer($anexos, $row['CDFILIRECE'], $row['NRLANBOLPAG']);
            }

            // Registra Dados
            $response->addDataSet(new DataSet('geraTituloAPagarDoc', ['nrlanbolpag' => $row['CDFILIBPAG'].$row['NRLANBOLPAG']]));
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Salva Anexo do Titulo a Pagar antes de salvar no FileServer
     * @param Row $row
     * @param Response $response
     */
    public function salvaAnexoTitulo(Row $row, Response $response)
    {
        try {
            
            $row  = $row->getRow();
            
            $this->titulosAPagarService->salvaAnexoTitulo($row);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Salva Anexo do Titulo a Pagar no file Server
     * @param Row $row
     * @param Response $response
     */
    public function salvaAnexoGrid(Row $row, Response $response)
    {
        try {
            
            $row  = $row->getRow();
            
            $this->titulosAPagarService->salvaAnexoGrid($row);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta Anexo do Titulo a Pagar antes de salvar no FileServer
     * @param Row $row
     * @param Response $response
     */
    public function deletaAnexoTitulo(Data $dataset, Response $response)
    {
        try {
            
            $rows = $dataset->getDataSet()->getRows();
            
            $this->titulosAPagarService->deletaAnexoTitulo($rows);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta Anexo do Titulo a Pagar e seu arquivo no FileServer
     * @param Row $row
     * @param Response $response
     */
    public function deleteAnexoGrid(Data $dataset, Response $response)
    {
        try {
            
            $rows = $dataset->getDataSet()->getRows();
            
            $this->titulosAPagarService->deleteAnexoGrid($rows);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta Anexo do Titulo a Pagar que restou como "lixo de memoria"
     * @param Row $row
     * @param Response $response
     */
    public function deleteAnexoIncompletos()
    {
        try {
            
            $this->titulosAPagarService->deleteAnexoIncompletos();
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Busca a url do FileServer
     * @param Row $row
     * @param Response $response
     */
    public function getFileServerUrl(Row $request, Response $response) {
        try {
            
            $fileName = $this->titulosAPagarService->getFileServerUrl();
            
            $response->addDataset(new DataSet('sucess', array("fileName" => $fileName)));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    
    /**
     * Habilita Titulo a Pagar
     * @param Data $dataset
     * @param Response $response
     */
    public function habilitaTituloAPagar(Data $dataset, Response $response){
        try {
            // Dados do Registro
            $rows  = $dataset->getDataSet()->getRows();

            // Declaração de Variáveis
            $boletos    = $rows[0];
            $idhabibpag = $rows[1][0];

            // Habilita Título a Pagar
            $this->titulosAPagarService->habilitaTituloAPagar($boletos, $idhabibpag);

            // Registra Dados
            $response->addDataSet(new DataSet('habilitaTituloAPagar', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    

    /**
     * Cadastrar Guia de Tributo
     * @param Data $dataset
     * @param Response $response
     */
    public function cadastrarGuiaTributo(Data $dataset, Response $response)
    {
        try {
            $dataset  = $dataset->getDataSet()->getRows();
            
            $dadosTributo       = $dataset[0];
            $titulosGuiaTributo = $dataset[1];

            $this->titulosAPagarService->cadastrarGuiaTributo($dadosTributo, $titulosGuiaTributo);
            
            $response->addDataSet(new DataSet('cadastraGuiaTributo', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    public function verificaDadosGuiaTributo(Row $row, Response $response){
        $row  = $row->getRow();
        $guiaCadastrada = $this->titulosAPagarService->verificaDadosGuiaTributo($row);
        $response->addDataset(new DataSet('dadosGuia', $guiaCadastrada));
    }
    

}
