<?php

namespace Teknisa\FIN\Modules\ContaPagar\Factory;

use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Ratctpagar;
use Teknisa\Model\Financeiro\Entities\Rattpctp;
use Teknisa\Model\Geral\Entities\Centcustfil;

/**
 * Class RateioContaFactory
 * @package Teknisa\FIN\Modules\ContaPagar\Factory
 */
class RateioContaFactory {

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    /**
     * Busca Rateio Conta Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @param $cdcentcust
     * @return Ratctpagar
     */
    public function buscaRateioConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdcentcust
    ){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanctonf']   = $nrlanctonf;
        $params['nrseqvenpag']  = $nrseqvenpag;
        $params['nrparcelapag'] = $nrparcelapag;
        $params['cdcentcust']   = $cdcentcust;

        /** @var Ratctpagar $ratctpagar */
        $ratctpagar = $this->entityManager->getRepository(Ratctpagar::class)->findOneBy($params);

        return $ratctpagar;
    }

    /**
     * Busca Rateios Conta Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @return Ratctpagar
     */
    public function buscaRateiosConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag
    ){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanctonf']   = $nrlanctonf;
        $params['nrseqvenpag']  = $nrseqvenpag;
        $params['nrparcelapag'] = $nrparcelapag;

        /** @var Ratctpagar $ratctpagar */
        $ratctpagar = $this->entityManager->getRepository(Ratctpagar::class)->findBy($params);

        return $ratctpagar;
    }


    /**
     * Busca Centro de Custo do Maior Valor Rateio Conta a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @param $cdfiratctpag
     * @return bool
     */
    public function buscaCentroCustoMaiorValorRateioConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdfiratctpag
    ){
        $connection = $this->entityManager->getConnection();

        $params                   = array();
        $params['P_NRORG']        = $nrorg;
        $params['P_CDFILIAL']     = $cdfilial;
        $params['P_NRLANCTONF']   = $nrlanctonf;
        $params['P_NRSEQVENPAG']  = $nrseqvenpag;
        $params['P_NRPARCELAPAG'] = $nrparcelapag;
        $params['P_CDFIRATCTPAG'] = $cdfiratctpag;

        $sql = "  SELECT MIN(CDCENTCUST) CDCENTCUST
                    FROM RATCTPAGAR
                    WHERE NRORG        = :P_NRORG
                      AND CDFILIAL     = :P_CDFILIAL
                      AND NRLANCTONF   = :P_NRLANCTONF
                      AND NRSEQVENPAG  = :P_NRSEQVENPAG
                      AND NRPARCELAPAG = :P_NRPARCELAPAG
                      AND CDFIRATCTPAG = :P_CDFIRATCTPAG
                      AND VRRATCCCTPAG = (SELECT MAX(VRRATCCCTPAG) VRRATCCCTPAG
                                                FROM RATCTPAGAR
                                               WHERE NRORG        = :P_NRORG
                                                 AND CDFILIAL     = :P_CDFILIAL
                                                 AND NRLANCTONF   = :P_NRLANCTONF
                                                 AND NRSEQVENPAG  = :P_NRSEQVENPAG
                                                 AND NRPARCELAPAG = :P_NRPARCELAPAG
                                                 AND CDFIRATCTPAG = :P_CDFIRATCTPAG)";

        $std = $connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    /**
     * Insere Rateio Conta a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @param $cdcentcust
     * @param $vrratccctpag
     * @return Ratctpagar
     */
    public static function insereRateioConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdfiratctpag, $cdcentcust, $vrratccctpag
    ){
        /** @var Ratctpagar $ratctpagar */
        $ratctpagar = New Ratctpagar();
        $ratctpagar->setNrorg($nrorg);
        $ratctpagar->setCdfilial($cdfilial);
        $ratctpagar->setNrlanctonf($nrlanctonf);
        $ratctpagar->setNrseqvenpag($nrseqvenpag);
        $ratctpagar->setNrparcelapag($nrparcelapag);
        $ratctpagar->setCdfiratctpag($cdfiratctpag);
        $ratctpagar->setCdcentcust($cdcentcust);
        $ratctpagar->setVrratccctpag($vrratccctpag);
        $ratctpagar->setDtinclusao(DateUtil::getDataAtual());

        return $ratctpagar;
    }

    /**
     * Atualiza Rateio Conta Pagar
     * @param Ratctpagar $ratctpagar
     * @param $vrratccctpag
     * @return Ratctpagar
     */
    public static function atualizaMaiorValorRateioConta(
        Ratctpagar $ratctpagar, $vrratccctpag
    ){
        $vrratccctpag += $ratctpagar->getVrratccctpag();
        $ratctpagar->setVrratccctpag($vrratccctpag);

        return $ratctpagar;
    }

    /**
     * Busca Rateio Conta Pagar
     * @param $nrorg
     * @param $nrlanctonf
     * @param $cdfilial
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @param $cdfilirece
     * @return Ratctpagar
     */
     public function buscaRatCtPagar($nrorg, $nrlanctonf, $cdfilial, $nrseqvenpag, $nrparcelapag, $cdfilirece){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['cdfiratctpag'] = $cdfilirece;
        $params['nrlanctonf']   = $nrlanctonf;
        $params['nrparcelapag'] = $nrparcelapag;
        $params['nrseqvenpag']  = $nrseqvenpag;

        /** @var Ratctpagar $ratctpagar */
        $ratctpagar = $this->entityManager->getRepository(Ratctpagar::class)->findBy($params);

        return $ratctpagar;
    }

    /**
     * Busca Rateio Tipo de Conta Pagar
     * @param $nrorg
     * @param $cdtpctpagar
     * @return Rattpctp
     */
    public function buscaRatTpCtP($nrorg, $cdtpctpagar){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdtpctpagar'] = $cdtpctpagar;

        /** @var Rattpctp $rattpctp */
        $rattpctp = $this->entityManager->getRepository(Rattpctp::class)->findBy($params);

        return $rattpctp;
    }

    /**
     * Busca Centro de Custo Filial
     * @param $nrorg
     * @param $cdfilial
     * @return Centcustfil
     */
    public function buscaCentCustFil($nrorg, $cdfilial){
        /** @var $params */
        $params = array();
        $params['nrorg']    = $nrorg;
        $params['cdfilial'] = $cdfilial;

        /** @var Centcustfil $centcustfil */
        $centcustfil = $this->entityManager->getRepository(Centcustfil::class)->findOneBy($params);

        return $centcustfil;
    }

    /**
     * Insere Rateio Tipo Conta Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $cdfilirece
     * @param $cdcentcust
     * @param $vrperattpctp
     * @return Rattpctp
     */
    public static function insereRatTpCtP(
        $nrorg, $cdfilial, $cdfilirece, $cdcentcust, $vrperattpctp
    ){
        /** @var Rattpctp $rattpctp */
        $rattpctr = New Rattpctp();
        $rattpctr->setNrorg($nrorg);
        $rattpctr->setCdfilial($cdfilial);
        $rattpctr->setCdfirattpctp($cdfilirece);
        $rattpctr->setCdcentcust($cdcentcust);
        $rattpctr->setVrperattpctp($vrperattpctp);
        $rattpctr->setDtinclusao(DateUtil::getDataAtual());

        return $rattpctr;
    }

    /**
     * Insere Rateio Conta Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @param $cdfiratctpag
     * @param $cdcentcust
     * @param $vrratccctpag
     * @return Ratctpagar
     */
    public static function insereRatCtPagar(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdcentcust, $cdfiratctpag, $vrratccctpag
    ){
        /** @var Ratctpagar $rateiopagar */
        $rateiopagar = new Ratctpagar();
        $rateiopagar->setNrorg($nrorg);
        $rateiopagar->setCdfilial($cdfilial);
        $rateiopagar->setNrlanctonf($nrlanctonf);
        $rateiopagar->setNrseqvenpag($nrseqvenpag);
        $rateiopagar->setNrparcelapag($nrparcelapag);
        $rateiopagar->setCdfiratctpag($cdfiratctpag);
        $rateiopagar->setCdcentcust($cdcentcust);
        $rateiopagar->setVrratccctpag($vrratccctpag);
        $rateiopagar->setDtinclusao(DateUtil::getDataAtual());

        return $rateiopagar;
    }

     /**
     * Atualiza Rateio Conta Pagar
     * @param Ratctpagar $ratctpagar
     * @param $vrratccctpag
     * @return Ratctpagar
     */
    public static function atualizaRatCtPagar(
        Ratctpagar $ratctpagar, $vrratccctpag
    ){
        $ratctpagar->setVrratccctpag($vrratccctpag);

        return $ratctpagar;
    }
}