<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Contapagar;
use Teknisa\Model\Financeiro\Entities\Movfluxo;
use Teknisa\Model\Financeiro\Entities\Ajctpagar;
use Teknisa\Model\Financeiro\Entities\Tipoajpagar;
use Teknisa\FIN\Modules\ContaPagar\Factory\ProrrogacaoTituloFactory;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;

/**
 * Class ProrrogacaoTituloService
 * @package Teknisa\FIN\Modules\ContaPagar\Services
 */
class ProrrogacaoTituloService {

    const BOLETO_ALTERADO = '3';
    const PERMITE_ALTERACAO_BOLETO = '2';

    protected $entityManager;
    protected $prorrogacaoTituloFactory;
    protected $fluxoDeCaixaService;

    /**
     * @param EntityManager $entityManager
     * @param ProrrogacaoTituloFactory $prorrogacaoTituloFactory
     * @param FluxoDeCaixaService $fluxoDeCaixaService
     */
    public function __construct(EntityManager $entityManager, ProrrogacaoTituloFactory $prorrogacaoTituloFactory, FluxoDeCaixaService $fluxoDeCaixaService)
    {
        $this->entityManager = $entityManager;
        $this->prorrogacaoTituloFactory = $prorrogacaoTituloFactory;
        $this->fluxoDeCaixaService = $fluxoDeCaixaService;
    }

    /**
     * Prorroga Título a Pagar
     * @param $nrorg
     * @param $dtvenbolpag
     * @param $cdmotipror
     * @param $boletos
     * @throws \Exception
     */
    public function prorrogaTituloAPagar($nrorg, $dtvenbolpag, $cdmotipror, $boletos, $idtppror, $nrdiasvenc, $cdreducont = null)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            
            $dtvenbolpag = $this->formataData($dtvenbolpag);
            
            // Percorre Boletos a Pagar
            foreach ($boletos as $boleto){

                // Declaração de Variáveis
                $cdfilial    = $boleto["CDFILIAL"];
                $nrlanbolpag = $boleto["NRLANBOLPAG"];
                
                // Recalcula Data - Prazo em Dias
                if($idtppror == 'P'){
                    $dtvenbolrecatual = $this->formataData($boleto['DTVENBOLPAG']);
                    $dtvenbolpag = DateUtil::adicionaIntervalo($dtvenbolrecatual, $nrdiasvenc);
                }
                
                // Prorroga Titulo a Pagar
                $this->geraProrrogacaoTitulo($nrorg, $cdfilial, $nrlanbolpag, $dtvenbolpag, $cdmotipror, null, $cdreducont);
                
            }
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Prorroga Título a Pagar
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $nrorg
     * @param $dtvenbolpag
     * @param $cdmotipror
     * @throws \Exception
     */
    public function geraProrrogacaoTitulo($nrorg, $cdfilial, $nrlanbolpag, $dtvenbolpag, $cdmotipror, $dtemisantecipa = null, $cdreducont = null)
    {
        try {
            /** @var Boletpagar $boleto */
            $boleto = $this->prorrogacaoTituloFactory->buscaBoletPagar(
                $nrorg, $cdfilial, $nrlanbolpag
            );

            // Formata Data de Vencimento[Prorrogação]
            $dtvenbolpag = $this->formataData($dtvenbolpag);

            // Valida Prorrogação Título
            $this->validaProrrogacaoBoleto($boleto, $dtvenbolpag, $dtemisantecipa);

            $boleto->setDtvenbolpag($dtvenbolpag);
            $boleto->setCdreducont($cdreducont);

            $this->entityManager->persist($boleto);

            //Atualiza Fluxo das Contas do Boleto

            /** @var Contapagar $contaspagar */
            //Busca Contas Associadas ao Título
            $contaspagar = $this->prorrogacaoTituloFactory->buscaContaspagarAssoc(
                $nrorg, $cdfilial, $nrlanbolpag
            );

            /** @var $conta Contapagar*/
            foreach($contaspagar as $conta){

                // Formata Data de Movimentação de Fluxo
                $dtmovfluxo = $conta->getDtmovfluxo()->format(DateUtil::FORMATO_BRASILEIRO);

                // Atualiza Fluxo de Caixa[RETIRADA]
                /** @var Movfluxo $movfluxo */
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $nrorg, $conta->getCdempresa(), $conta->getCdfimovflpag(), $conta->getCdclassfina(), $dtmovfluxo, $conta->getVrorigpag()*-1, $conta->getIdmovfluxo(), 'S'
                );

                $this->entityManager->persist($movfluxo);
                $this->entityManager->flush();

                // Atualiza Fluxo de Caixa[INSERÇÃO]
                /** @var Movfluxo $movfluxo */
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $nrorg, $conta->getCdempresa(), $conta->getCdfimovflpag(), $conta->getCdclassfina(), $dtvenbolpag, $conta->getVrorigpag(), $conta->getIdmovfluxo(), 'S'
                );

                $this->entityManager->persist($movfluxo);
                $this->entityManager->flush();

                $conta->setDtmovfluxo($dtvenbolpag);
                $conta->setDtatuavenpag($dtvenbolpag);
                $conta->setCdmotipror($cdmotipror);
                $conta->setIdmovfluxo('E');

                $this->entityManager->persist($conta);

                //Atualiza Fluxo dos Ajustes
                /** @var Ajctpagar $ajustes */
                $ajustes = $this->prorrogacaoTituloFactory->buscaAjsctpagar(
                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf(),
                    $conta->getNrseqvenpag(), $conta->getNrparcelapag()
                );

                /** @var  $ajuste Ajctpagar*/
                foreach ($ajustes as $ajuste){

                    // Busca Tipo de Ajuste Pagar
                    /** @var  $tipoajuste Tipoajpagar*/
                    $tipoajuste = $this->prorrogacaoTituloFactory->buscaTipoajpagar(
                        $ajuste->getNrorg(),
                        $ajuste->getCdajuctpag()
                    );

                    // Verifica Tipo de Ajuste Pagar
                    if($tipoajuste->getIdajuctpag() == 'D'){
                        // Atualiza Fluxo de Caixa[INSERÇÃO]
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(), $ajuste->getCdclassfina(), $ajuste->getDtmovfluxo(), $ajuste->getVrajustepag(), 'E', 'S'
                        );
                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();

                        // Atualiza Fluxo de Caixa[Remoção]
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(), $ajuste->getCdclassfina(), $dtvenbolpag, $ajuste->getVrajustepag()*-1, 'E', 'S'
                        );
                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();
                    }else{
                        // Atualiza Fluxo de Caixa[Remoção]
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(), $ajuste->getCdclassfina(), $ajuste->getDtmovfluxo(), $ajuste->getVrajustepag()*-1, 'E', 'S'
                        );
                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();


                        // Atualiza Fluxo de Caixa[INSERÇÃO]
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $ajuste->getCdemmfajcntp(), $ajuste->getCdfimfajcntp(), $ajuste->getCdclassfina(), $dtvenbolpag, $ajuste->getVrajustepag(), 'E', 'S'
                        );
                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();
                    }

                    $ajuste->setDtmovfluxo($dtvenbolpag);

                    $this->entityManager->persist($ajuste);
                }
            }

            $this->entityManager->flush();
        }catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * Valida Prorrogação Título
     * @param Boletpagar $boletpagar
     * @param $dtvenbolpag
     * @throws \Exception
     */
    public function validaProrrogacaoBoleto(Boletpagar $boletpagar, $dtvenbolpag, $dtemisantecipa = null)
    {
        try{
            // Declara Menor Data Válida do Sistema
            $menordata = $dtemisantecipa?$this->formataData($dtemisantecipa):$this->formataData('31/12/2037');
    
            /** @var Contapagar $contaspagar */
            // Busca Contas a Pagar
            $contaspagar = $this->prorrogacaoTituloFactory->buscaContaspagarAssoc(
                $boletpagar->getNrorg(), $boletpagar->getCdfilial(), $boletpagar->getNrlanbolpag()
            );
    
            /** @var Contapagar $contapagar */
            // Percorre Contas a Pagar
            foreach($contaspagar as $contapagar){
    
                /** @var  $nota Notafiscal */
                // Busca Nota Fiscal
                $nota = $this->prorrogacaoTituloFactory->buscaNotafiscal(
                    $contapagar->getNrorg(), $contapagar->getCdfilial(), $contapagar->getNrlanctonf()
                );
    
                // Compara Datas do
                if(DateUtil::comparaDatas($nota->getDtemissao(), $menordata) && !$dtemisantecipa)
                    $menordata = $nota->getDtemissao();
            }
            
            if(DateUtil::comparaDatas($dtvenbolpag, $menordata))
                throw new \Exception('Data de vencimento deve ser maior ou igual a data de emissão da nota fiscal.');
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * Formata Data
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public static function formataData(
        $data
    ){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

}