<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco;

use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\Emprbanc;
use Financeiro\Logic\ContasReceber\Cnab\Remessa\GeraCnab;
use Geral\Entities\Banco;
use Geral\Entities\Endeclie;
use Geral\Entities\Licencauso;
use Geral\Entities\Contcorr;
use Geral\Entities\Empresa;
use Teknisa\Libs\Util\DateUtil;
use Financeiro\Util\Util;
use Geral\Entities\Cliente;
use Geral\Entities\Filial;
use Geral\Entities\Municipio;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\GeraCnabService;

class GeraCnabSofisaService extends GeraCnabService{

    const IDENTIFICACAO_REGISTRO = '0';
    const IDENTIFICACAO_ARQUIVO_REMESSA = '1';
    const LITERAL_REMESSA = 'REMESSA';
    const CODIGO_SERVICO = '05';
    const LITERAL_SERVICO = 'COBRANCASIMPLES';
    const NUMERO_SOFISA = '999';
    const IDENTIFICACAO_SISTEMA = 'MX';
    const IDENTIFICACAO_DESCONTO = 'DESC';
    const NUMERO_SEQUENCIAL_ARQUIVO = '1';
    const IDENTIFICACAO_REGISTRO_TRAILLER = '9';
    const IDENTIFICACAO_REGISTRO_TRANSACAO = '1';
    const VERSAO_LAYOUT = 'v.2.6';
    
    
    public function geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnab, $cdreducont, $carteira = NULL, $contreg = NULL, $idtipcnabrec = NULL, $cdfilial, $contador = NULL){
        try {
            
            $idtipcnabrec = $cdidrec;
            
            if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
            $nmarquivo = $caminho.$nmarquivo;
            $file = fopen($nmarquivo, "w+");

            //#################################################################//
            //############### REGISTRO HEADER DE ARQUIVO ######################//
            //#################################################################//

            // CAMPO: Identificação do Registro
            // POSIÇÃO: 001 a 001
            // PICTURE: 9(1)
            $arquivo = self::IDENTIFICACAO_REGISTRO;
            
            // CAMPO: TIPO DE OPERAÇÃO – REMESSA
            // POSIÇÃO: 002 a 002
            // PICTURE: 9(1)
            $arquivo .= self::IDENTIFICACAO_ARQUIVO_REMESSA;
            
            // CAMPO: NOME POR EXTENSO DO TIPO DE MOVIMENTO
            // POSIÇÃO: 003 a 009
            // PICTURE: X(7)
            $arquivo .= self::LITERAL_REMESSA;
            
            // CAMPO: IDENTIFICAÇÃO DO PRODUTO
            // POSIÇÃO: 010 a 011
            // PICTURE: 9(2)
            $arquivo .= self::CODIGO_SERVICO;
            
            // CAMPO: IDENTIFICAÇÃO POR EXTENSO DO PRODUTO
            // POSIÇÃO: 012 a 026
            // PICTURE: X(15)
            $arquivo .= self::LITERAL_SERVICO;
            
            // CAMPO: AGÊNCIA MANTEDORA DA CONTA/COMPLEMENTO DE REGISTRO/NÚMERO DA CONTA DA EMPRESA/COMPLEMENTO DE REGISTRO
            // POSIÇÃO: 027 a 046
            // PICTURE: 9(20)
            $idenRec = $this->getIdContRec($cdreducont, $cdbanco, $cdempresa);
            if($idenRec){
                $cdidreccontcorr = str_pad($idenRec, 20);
                $arquivo .= Util::campoCnabValido($cdidreccontcorr);
            }else $arquivo = str_pad($arquivo, 20);
            
            // CAMPO: NOME POR EXTENSO DA EMPRESA
            // POSIÇÃO: 047 a 076
            // PICTURE: X(30)
            /** @var $empresa Empresa */
            $empresa = Empresa::findOneByCdempresa($cdempresa);
            $filial = Filial::findOneByCdfilial($cdfilial);
            
            // Caso seja informado filial na geração do CNAB, sera inserido nome da razão social da filial e não da empresa no arquivo
            if($filial) {
                $nmrazsocemp = strtoupper($filial->getNmfilial());
                $nmrazsocemp = substr($nmrazsocemp, 0, 30);
                $arquivo .= str_pad($nmrazsocemp, 30);
            } else {
                $nmrazsocemp = strtoupper($empresa->getNmempresa());
                $nmrazsocemp = substr($nmrazsocemp, 0, 30);
                $arquivo .= str_pad($nmrazsocemp, 30);
            }
            
            // CAMPO: CÓDIGO DA INSTITUIÇÃO
            // POSIÇÃO: 077 a 079
            // PICTURE: 9(3)
            $arquivo .= self::NUMERO_SOFISA;
            
            // CAMPO: NOME DA INSTITUIÇÃO
            // POSIÇÃO: 080 a 094
            // PICTURE: X(15)
            $banco = Banco::findOneByCdbanco($cdbanco);
            $arquivo .= str_pad(strtoupper($banco->getNmbanco()), 15);
            
            // CAMPO: DATA DE GERAÇÃO DO ARQUIVO
            // POSIÇÃO: 095 a 100
            // PICTURE: 9(6)
            $now = DateUtil::getDataAtual();
            $data = DateUtil::getDia($now) . DateUtil::getMes($now) . substr(DateUtil::getAno($now), -2);
            $arquivo .= Util::campoCnabValido($data);
            
            // CAMPO: COMPLEMENTO DE REGISTRO
            // POSIÇÃO: 101 a 380
            // PICTURE: X(280)
            $arquivo = str_pad($arquivo, 380);
            
            // CAMPO: VERSÃO DO LAYOUT
            // POSIÇÃO: 381 a 385
            // PICTURE: X(5)
            $arquivo .= self::VERSAO_LAYOUT;

            // CAMPO: Nº DE IDENTIFICAÇÃO ARQUIVO REMESSA
            // POSIÇÃO: 386 a 394
            // PICTURE: 9(9)
            $arquivo .= substr($nrseqremessarec, -9);
            
            // CAMPO: Nº SEQUENCIAL DE REGISTRO NO ARQUIVO
            // POSIÇÃO: 395 a 400
            // PICTURE: 9(6)
            $arquivo .= str_pad($contreg + 1, 6, "0", STR_PAD_LEFT) . "\r\n";
            //Grava a variavel arquivo no arquivo texto
            fprintf($file, "%s", $arquivo);
            fclose($file);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraTransacao(
         $caminho, $nmarquivo,$cdempresa,$nrorg, $cdcartcobbol, $cdagencia, $cdcontcorr, $cdfilial,
        $nrlanbolrec, $vrpemobrec, $idimprboleto, $cdnosnumrec, $dtvenbolrec, $cdins1cobbol,
        $qtdiaprotbol, $vrmordiabrec, $vrfinalbrec, $idopecnabrec, $vrpemulbrec, $idtpijurclie,
        $nrinsjurclie, $cdcliente, $cdins2cobbol, $contreg = NULL, $idtipcnabrec = null, $idfilialcnabrec, $cddiaprotbol = NULL, $contador = NULL,$cdreducont,$cdbanco
    ){
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Identificação do Registro
        // POSIÇÃO: 001 a 001
        // PICTURE: 9(1)
        $arquivo = self::IDENTIFICACAO_REGISTRO_TRANSACAO;
        $empresa = Empresa::findOneByCdempresa($cdempresa);
        $filial = Filial::findOneByCdfilial($idfilialcnabrec);
        
        if($filial){
            // CAMPO: Tipo de Inscrição da Filial
            // POSIÇÃO: 002 a 003
            // PICTURE: 9(2)
           
            switch ($filial->getIdtpijurfili()){
                case 'F':
                    $arquivo .= '01';
                    break;
                case 'J':
                    $arquivo .= '02';
                    break;
                case 'I':
                case 'O':
                    $arquivo .= '03';
                    break;
            }
            
            // CAMPO: Número de Inscrição da Filial
            // POSIÇÃO: 004 a 017
            // PICTURE: 9(14)
            $arquivo .= substr($filial->getNrinsjurfili(), 0, 14);
        } else {
            
            // CAMPO: Tipo de Inscrição da Filial
            // POSIÇÃO: 002 a 003
            // PICTURE: 9(2)
            
            switch ($empresa->getIdtpijurempr()){
                case 'F':
                    $arquivo .= '01';
                    break;
                case 'J':
                    $arquivo .= '02';
                    break;
            }
            
            // CAMPO: Número de Inscrição da Empresa
            // POSIÇÃO: 004 a 017
            // PICTURE: 9(14)
            $arquivo .= substr($empresa->getNrinsjurempr(), 0, 14);
        }
        
        // CAMPO: Identificação da empresa no banco
        // POSIÇÃO,: 018 a 037
        // PICTURE: X(20)
        $idenRec = $this->getIdContRec($cdreducont, $cdbanco, $cdempresa);
        if($idenRec){
            $cdidreccontcorr = str_pad($idenRec, 20);
            $arquivo .= Util::campoCnabValido($cdidreccontcorr);
        }else $arquivo = str_pad($arquivo, 20);
        
        // CAMPO: Identificação do título na empresa
        // POSIÇÃO: 038 a 062
        // PICTURE: X(25)
        $arquivo .= str_pad($cdfilial.$nrlanbolrec,25);
        
        // CAMPO: Identificação do Título no Banco - nosso numero
        // POSIÇÃO: 063 a 079
        // PICTURE: 9(17)
        //VERIFICAR
        if($idimprboleto && $idimprboleto == 'E'){
            $arquivo .= str_pad($cdnosnumrec,17,'0');
        }else {
            $arquivo .= str_pad($cdnosnumrec,17, '0');
        }

        // CAMPO: Complemento de Registro
        // POSIÇÃO: 80 a 82
        // PICTURE: x(3)
        $arquivo .= self::preenche_brancos('', 3);
        
        // CAMPO: responsavel pela impressao do boleto
        // POSIÇÃO: 83
        // PICTURE: x(1)
        if(($idimprboleto == 'E') || ($idimprboleto == 'F')){
            $arquivo .= '2';
        } else{
            $arquivo .= '1';
        }
        
        // CAMPO: Número da carteira na factoring
        // POSIÇÃO: 084 a 086
        // PICTURE: x(3)
        $arquivo .= '001';
    
        // CAMPO: IDENTIFICAÇÃO DO TÍTULO NA FACTORING
        // POSIÇÃO: 087 a 094
        // PICTURE: 9(8)
        $arquivo .= str_repeat('0',8);
        
        // CAMPO: COMPLEMENTO DE REGISTRO
        // POSIÇÃO: 095 a 108
        // PICTURE: x(14)
        $arquivo .= self::preenche_brancos('', 14);
        
        // CAMPO: Identificação da ocorrência
        // POSIÇÃO: 109 a 110
        // PICTURE: 9(2)
        $arquivo .= '01';
        
        // CAMPO: Nº do documento de cobrança
        // POSIÇÃO: 111 a 120
        // PICTURE: X(10)
        $entitieDup = Boletreceber::getRepository();
        $duplicata = $entitieDup->getDuplicata($cdfilial, $nrlanbolrec);
        if($duplicata)
            $arquivo .= substr($duplicata[0]['DUPLICATA'], 0, 8) .'/'. $duplicata[0]['NRPARCELA'];
        else
            $arquivo = str_pad($arquivo, 10);

        // CAMPO: Data de vencimento do título
        // POSIÇÃO: 121 a 126
        // PICTURE: 9(6)
        $vencimento = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
        $vencimento = DateUtil::getDia($vencimento) . DateUtil::getMes($vencimento) . substr(DateUtil::getAno($vencimento), -2);
        $arquivo .= Util::campoCnabValido($vencimento);
        
        // CAMPO: Valor do título
        // POSIÇÃO: 127 a 139
        // PICTURE: 9(11)V9(2)
        $vr = $vrfinalbrec * 100;
        $vr = str_pad($vr, 13, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($vr);
        
        // CAMPO: COMPLEMENTO DE REGISTRO
        // POSIÇÃO: 140 a 147
        // PICTURE: 9(8)
        $arquivo .= str_repeat('0',8);
        
        // CAMPO: Espécie do título
        // POSIÇÃO: 148 a 149
        // PICTURE: 9(2)
        $arquivo .= '01';
        
        // CAMPO: COMPLEMENTO DE REGISTRO
        // POSIÇÃO: 150
        // PICTURE: x(1)
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Data de emissão do título
        // POSIÇÃO: 151 a 156
        // PICTURE: 9(6)
        $now = DateUtil::getDataAtual();
        $data = DateUtil::getDia($now) . DateUtil::getMes($now) . substr(DateUtil::getAno($now), -2);
        $arquivo .= Util::campoCnabValido($data);
        
        // CAMPO: 1ª instrução
        // POSIÇÃO: 157 a 158
        // PICTURE: 9(2)
        if($cdins1cobbol)
            $arquivo .= $cdins1cobbol;
        else
            $arquivo .= '00';
        
        // CAMPO: 2ª instrução
        // POSIÇÃO: 159 a 160
        // PICTURE: 9(2)
        if($cdins2cobbol)
            $arquivo .= $cdins2cobbol;
        else
            $arquivo .= '00';
        
        // CAMPO: Valor de mora por dia de atraso
        // POSIÇÃO: 161 a 173
        // PICTURE: 9(11)V9(2)
        if(!$vrmordiabrec){
            if(!$vrpemobrec){
                $arquivo = str_pad($arquivo, 173, '0');
            }
            else{
                $arquivo .= str_pad(str_replace('.', '', ($vrfinalbrec * $vrpemobrec) / 100), 13, '0', STR_PAD_LEFT);
            }
        }
        else{
            $arquivo .= str_pad(str_replace('.', '', $vrmordiabrec * 100), 13, '0', STR_PAD_LEFT);
        }
        
        // CAMPO: Data limite p/ concessão de desconto
        // POSIÇÃO: 174 a 179
        // PICTURE: 9(6)
        $arquivo .= '000000';
        
        // CAMPO: Valor do Desconto
        // POSIÇÃO: 180 a 192
        // PICTURE: 9(11)V9(2)
        $arquivo = str_pad($arquivo, 192, '0');
        
        // CAMPO: Valor do I.O.F.
        // POSIÇÃO: 193 a 205
        // PICTURE: 9(11)V9(2)
        $arquivo = str_pad($arquivo, 205, '0');
        
        // CAMPO: Valor do Abatimento a ser concedido
        // POSIÇÃO: 206 a 218
        // PICTURE: 9(11)V9(2)
        $arquivo = str_pad($arquivo, 218, '0');
        
        $nrInsJur = 0;
        $nrCepClie = 0;
        $nrInsJur = str_pad($nrInsJur, 14, '0');
        $nrCepClie = str_pad($nrCepClie, 8, '0');
        
        $entitieCliente = Cliente::findOneByCdcliente($cdcliente);
       
        if($entitieCliente){
            if($entitieCliente->getIdtpijurclie() == 'F')
                $tpInscSacado = '01';
            else
                $tpInscSacado = '02';

            if ($entitieCliente->getNrinsjurclie())
                $nrInsJur = $entitieCliente->getNrinsjurclie();
            /** @var $entitieEndeClie Endeclie */
            $entitieEndeClie = Endeclie::findOneByCdclienteAndIdtpendeclie($cdcliente, 'C');

            if($entitieEndeClie){
                if($entitieEndeClie->getNrcepclie())
                    $nrCepClie = $entitieEndeClie->getNrcepclie();
                $nmClie = $entitieCliente->getNmrazsocclie();
                $dsEndeclie = $entitieEndeClie->getDsendeclie();
                $nmBairro = $entitieEndeClie->getNmbairclie();
                $municipio = Municipio::findOneByNrorgAndCdmunicipio($nrorg, $entitieEndeClie->getCdMunicipio());
                $nmMunicipio = $municipio->getNmmunicipio();
                $sgEstado = $entitieEndeClie->getSgestado();
            }
        }
        
        // CAMPO: Código de inscrição/Sacado ('01' = CPF, '02' = CGC)
        // POSIÇÃO: 219 a 220
        // PICTURE: 9(2)
        $arquivo .= $tpInscSacado;
        
        // CAMPO: Nº de inscrição do sacado (CPF/CGC)
        // POSIÇÃO: 221 a 234
        // PICTURE: 9(14)
        $arquivo .= Util::campoCnabValido(str_pad($nrInsJur, 14, '0', STR_PAD_LEFT));
        
        // CAMPO: Nome do sacador ou avalista
        // POSIÇÃO: 352 a 369
        // PICTURE: X(30)
        $nomecliente = substr(Util::campoCnabValido($nmClie), 0, 40);
        $arquivo .= str_pad($nomecliente, 40);
        
        // CAMPO: Logradouro (Endereço do sacado (cliente))
        // POSIÇÃO: 275 a 314
        // PICTURE: X(40)
        $enderecoCliente = substr(Util::campoCnabValido($dsEndeclie), 0, 40);
        $arquivo .= str_pad($enderecoCliente, 40);

        // CAMPO: Bairro do sacado
        // POSIÇÃO: 315 a 326
        // PICTURE: X(12)
        $bairro = substr(Util::campoCnabValido($nmBairro), 0, 12);
        $arquivo .= str_pad($bairro, 12);

        // CAMPO: C.E.P. do sacado
        // POSIÇÃO: 327 a 334
        // PICTURE: 9(8)
        $arquivo .= Util::campoCepTel($nrCepClie);
        
        // CAMPO: Cidade do sacado
        // POSIÇÃO: 335 a 349
        // PICTURE: X(15)
        $municipio = substr(Util::campoCnabValido($nmMunicipio), 0, 15);
        $arquivo .= str_pad($municipio, 15);

        // CAMPO: Estado do sacado
        // POSIÇÃO: 350 a 351
        // PICTURE: X(2)
        $arquivo .= Util::campoCnabValido(Util::campoCnabValido($sgEstado));
        
        // CAMPO: Nome do sacador ou avalista
        // POSIÇÃO: 352 a 369
        // PICTURE: X(18)
        $arquivo .= self::preenche_brancos('', 18);
        
        // CAMPO:  CÓDIGO DE INSCRIÇÃO DO SACADOR/AVALISTA
        // POSIÇÃO: 370 a 371
        // PICTURE: 9(2)
        $arquivo .= str_repeat('0', 2);
        
        // CAMPO:  CÓDIGO DE INSCRIÇÃO DO SACADOR/AVALISTA
        // POSIÇÃO: 372 a 385
        // PICTURE: 9(14)
        $arquivo .= str_repeat('0', 14);
        
        // CAMPO:  COMPLEMENTO DE REGISTRO
        // POSIÇÃO: 386 a 393
        // PICTURE: 9(8)
        $arquivo .= str_repeat('0', 8);
        
        // CAMPO:  COMPLEMENTO DE REGISTRO
        // POSIÇÃO: 394 a 394
        // PICTURE: X(1)
        $arquivo .= self::preenche_brancos('', 1);
        
        // CAMPO: Nº Seqüencial do Registro
        // POSIÇÃO: 395 a 400
        // PICTURE: 9(6)
        $arquivo .= str_pad($contreg, 6, "0", STR_PAD_LEFT) . "\r\n";
        
        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    public function geraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote = NULL, $contreg = NULL, $cdreducont = NULL){
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Identificação do registro
        // POSIÇÃO: 001 a 001
        // PICTURE: 9(1)
        $arquivo = self::IDENTIFICACAO_REGISTRO_TRAILLER;

        // CAMPO: Branco
        // POSIÇÃO: 002 a 394
        // PICTURE: X(394)
        $arquivo = str_pad($arquivo, '394');

        // CAMPO: Nº Seqüencial do Registro
        // POSIÇÃO: 395 a 400
        // PICTURE: 9(6)
        $arquivo .= str_pad($contreg , 6, "0", STR_PAD_LEFT). "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    private static function getCodigoEmpresa($cdbanco, $cdempresa, $cdidrec){
        $resultado = '';
        /** @var $banco Banco */
        $banco = Banco::findOneByCdbanco($cdbanco);
        $banccnab = $banco->getCdbanccnab();

        if($banccnab == '001'){
            /** @var $emprbanc Emprbanc */
            $emprbanc = Emprbanc::findOneByCdbancoAndCdempresa($cdbanco, $cdempresa);
        }

        if($cdidrec)
            $resultado = $cdidrec;

        else{
            if($banccnab != '001')
                /** @var $emprbanc Emprbanc */
                $emprbanc = Emprbanc::findOneByCdbancoAndCdempresa($cdbanco, $cdempresa);

            if($emprbanc->getCdempbancrec()){
                $resultado = $emprbanc->getCdempbancrec();
            }
        }
        return $resultado;
    }

     
    /**
     * @param $valor - valor a ser alterado
     * @param $tamanho - tamanho total da string desejado
     * @return string
    */
     
    private function preenche_brancos($valor, $tamanho){
        return str_pad($valor, $tamanho);
    }
    
    public function geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec, $cdreducont, $carteira = NULL, $cdfilial){}
    public function geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec,
        $nrcnabrec, $idoperacao, $cdagenciacnab, $cdcontacorrente, $cdcontcorrcobranca, $nossonumero,
        $idtipocobranca, $nrlanbolrec, $cdfilial, $dtvenbolrec, $valorfinalboleto, $valorpemobrec, $valormordiabrec,
        $valorpemulbrec, $valormultabrec, $valorfinalbrec, $cdcartcobbol, $cddiaprotbol, $cdreducont, $imprbol,
        $qtdiaprotbol, $contador = NULL){}
    public function geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $cdcliente, $idoperacao, $cdempresa, $cdfilial, $nrlanbolrec, $contador = NULL){}
    public function geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL){}
    public function GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco, $contreg = NULL){}
}